#define BSM_2K       0
#define BSM_CV       1
#define BSM_4K       2
#define BSM_F8SC     3
#define BSM_F8       4
#define BSM_FE       5
#define BSM_3F       6
#define BSM_E0       7
#define BSM_FANR     8
#define BSM_FA       9
#define BSM_F6SC    10
#define BSM_F6      11
#define BSM_E7      12
#define BSM_E7NR    13
#define BSM_F4SC    14
#define BSM_F4      15
#define BSM_MB      16

#define METER_LENGTH 32

#define DISPLAY_NOTHING  0
#define DISPLAY_FILENAME 1
#define DISPLAY_SUMMARY  2
#define DISPLAY_DETAILS  3

#ifdef _WIN32
#include "find.h"
#endif

#ifdef LINUX
#define _MAX_FNAME 1024
#define _MAX_PATH 1024
#define _MAX_DIR 1024
#define _MAX_DRIVE 0
#define _MAX_EXT 1024
#define stricmp strcasecmp
#endif

typedef unsigned long int	dd;
typedef unsigned short int	dw;
typedef int			  		db;


struct parmFlags
{
    int     dflag;
    int     fflag;
	int     kflag;
    int     mflag;
    int     pflag;
    int     sflag;
	int     vflag;
    char    tflag;
	int		zeroBitLength;
	int		oneBitLength;
    int     control;
    int     createMode;
    int     binFileCount;
	int     bankSwitchMode;
    float   headerSeconds;
	float	clearingSeconds;
    float   hlength;
	char	wavFileName[_MAX_FNAME];
	char	fullPath[_MAX_PATH];

	struct   qnode *binFileQ;
    unsigned int  multi;
    unsigned int  volume;
    unsigned char startHi;
    unsigned char startLo;
};

char bankSwitchList[][5] =
    {"2K",  "CV",   "4K",   "F8SC", "F8",	"FE",
     "3F",  "E0",   "FANR", "FA",   "F6SC", "F6",
     "E7",  "E7NR", "F4SC", "F4",   "MB",   ""};

unsigned int controlByteList[] =
    {0xCa,  0xEa,   0xC8,   0xE6,   0xC6,   0xCC,
     0xCE,  0xC1,   0xC0,   0xE0,   0xE4,   0xC4,
     0xE3,  0xC3,   0xE2,   0xC2,   0xC9,   0x00};

int fileSizeList[] =
    {2048,  2048,   4096,   8192,   8192,   8192,
     8192,  8192,   8192,   12288,  16384,  16384,
     16384, 16384,  32768,  32768,  65536,  0};

int startPageList[] =
    {7,     7,      15,     15,     15,     15,
     31,    31,     15,     15,     15,     15,
     63,    63,     15,     15,     15,     0};

unsigned char clearingTone[2][120] =
    {	/* can not contain a zero */
        {128, 157, 186, 210, 230, 244, 252, 252, 244, 230, 210, 186, 157,
         128,  98,  69,  45,  25,  11,   3,   3,  11,  25,  45, 69, 98, '\0'},

        {128, 143, 158, 173, 187, 200, 212, 222, 232, 239, 245, 250, 252,
         252, 251, 248, 243, 236, 227, 217, 206, 193, 180, 165, 150, 135,
         120, 105,  90,  75,  62,  49,  38,  28,  19,  12,   7,   4,   3,
           3,   5,  10,  16,  23,  33,  43,  55,  68,  82,  97, 112, '\0'}
    };

/* bit[speed][khz][byte] */

unsigned char zeroBit[81],
              oneBit[81];

unsigned char zeroBitList[5][2][80]=
	{ /* can not contain a 0 */
        {	/* Slow */
			{	/* 22khz (5/9) */
				0x06,0x06,0x80,0xfc,0xfc,'\0'
			},
			{	/* 44khz (15/21) */
				0x80,0x4d,0x23,0x09,0x03,0x13,0x36,0x66,
				0x99,0xc9,0xec,0xfc,0xf6,0xdc,0xb2,'\0'
			}
		},
		{	/* Fast */
			{	/* 22khz (3/7) */
				0x06,0x80,0xfc,'\0'
			},
			{	/* 44khz (6/10) */
				0x06,0x06,0x06,0xfc,0xfc,0xfc,'\0'
			}
		},
		{	/* Super Fast */
			{	/* 22khz (3/5) */
				0x06,0x80,0xfc,'\0'
			},
			{	/* 44khz (5/8) */
				0x06,0x06,0x80,0xfc,0xfc,'\0'
			}
		},
		{	/* Super Duper Fast */
			{	/* 22khz (3/5) */
				0x06,0x80,0xfc,'\0'
			},
			{	/* 44khz (4/8) */
				0x06,0x06,0xfc,0xfc,'\0'
			}
		},
		{	/* Lightning */
			{	/* 22khz (3/5) */
				0x06,0x80,0xfc,'\0'
			},
			{	/* 44khz (4/7) */
				0x06,0x06,0xfc,0xfc,'\0'
			}
		}
    };

unsigned char oneBitList[5][2][81]=
	{ /* can not contain a 0 */
		{	/* Slow */
			{	/* 22khz (5/9) */
				0x06,0x06,0x06,0x06,0x80,0xfc,0xfc,0xfc,
				0xfc,'\0'
			},
			{	/* 44khz (15/21) */
				0x80,0x5b,0x39,0x1e,0x0b,0x03,0x06,0x13,
				0x2a,0x49,0x6d,0x92,0xb6,0xd5,0xec,0xf9,
				0xfc,0xf4,0xe1,0xc6,0xa4,'\0'
			}
		},  
		{	/* Fast */
			{	/* 22khz (3/7) */
				0x06,0x06,0x06,0x80,0xfc,0xfc,0xfc,
				'\0'
			},
			{	/* 44khz (6/10) */
				0x06,0x06,0x06,0x06,0x06,0xfc,0xfc,0xfc,
				0xfc,0xfc,'\0'
			}
		},  
		{	/* Super Fast */
			{	/* 22khz (3/5) */
				0x06,0x06,0x80,0xfc,0xfc,
				'\0'
			},
			{	/* 44khz (5/8) */
				0x06,0x06,0x06,0x06,0xfc,0xfc,0xfc,0xfc,
				'\0'
			}
		},  
		{	/* Super Duper Fast */
			{	/* 22khz (3/5) */
				0x06,0x06,0x80,0xfc,0xfc,
				'\0'
			},
			{	/* 44khz (4/8) */
				0x06,0x06,0x06,0x06,0xfc,0xfc,0xfc,0xfc,
				'\0'
			}
		},  
		{	/* Lightning */
			{	/* 22khz (3/5) */
				0x06,0x06,0x80,0xfc,0xfc,
				'\0'
			},
			{	/* 44khz (4/7) */
				0x06,0x06,0x06,0x80,0xfc,0xfc,0xfc,
				'\0'
			}
		}
    }; 

unsigned char header[2][80]=
    {
        {'R','I','F','F',0x80,0x80,0x80,0x80,'W','A','V','E','f','m','t',' ',0x10,0,0,0,1,0,1,0,0x22,0x56,0,0,0x22,0x56,0,0,1,0,8,0,'d','a','t','a',0,0,0,0,'\0' },
        {'R','I','F','F',0x80,0x80,0x80,0x80,'W','A','V','E','f','m','t',' ',0x10,0,0,0,1,0,1,0,0x44,0xac,0,0,0x44,0xac,0,0,1,0,8,0,'d','a','t','a',0,0,0,0,'\0' }
    };

dd	BS_1[] = {
	0x34ae2945,	/* Magicard (CommaVid).bin */
/*	0x30eb4f7a,	   Video Life (4K).BIN */
/*	0x9afa761f,	   Magicard (Life).bin */
	0x266bd1b6,	/* Video Life (CommaVid).bin */
	-1
};

char BS_1_DESC[][25] = {
	"Magicard (CommaVid)",
	"Video Life (CommaVid)",
	-1
};

dd	BS_3[] = {
	0x2843d776,	/* Frogger II - Threedeep!.bin */
	0x690ada72,	/* Gyruss [b].bin */
	0x525ee7e9,	/* Gyruss.bin */
	0x95da4070,	/* James Bond 007 [b].bin */
	0x3216c1bb,	/* James Bond 007.bin */
	0xae4114d8,	/* Montezuma's Revenge.bin */
	0x00e44527,	/* Mr. Do!'s Castle.bin */
	0xf723b8a6,	/* Popeye.bin */
	0xe44c244e,	/* Q-bert's Qubes [a].bin */
	0xb8f2dca6,	/* Q-bert's Qubes.bin */
	0xe77f6742,	/* Star Wars - Death Star Battle (Parker Bros).bin */
	0xce09fcd4,	/* Star Wars - The Arcade Game (Parker Bros).bin */
	0xdd85f0e7,	/* Super Cobra [b].bin */
	0x8d372730,	/* Super Cobra.bin */
	0xd9088807,	/* Tooth Protectors (DSD-Camelot).bin */
	0x7eed7362,	/* Tutankham.bin */
	0xc87fc312,	/* Popeye_(eks).bin */
	0xef3ec01e,	/* Super Cobra_(eks).bin */
	0x84a101d4,	/* Star Wars - Death Star Battle_(eks).bin */
	0x2fc06cb0,	/* Tutankham_(eks).bin */
	0xab50bf11,	/* Star Wars - The Arcade Game (proto).BIN */
    0x549a1b6b, /* Star Wars - The Arcade Game (PAL) */
    0x36910e4d, /* Frogger II - Threedeep! (PAL) */
    0xb8bb2361, /* Gyruss (PAL) */
	-1
};

char BS_3_DESC[][50] = {
	"Frogger II - Threedeep!",
	"Gyruss [b]",
	"Gyruss",
	"James Bond 007 [b]",
	"James Bond 007",
	"Montezuma's Revenge",
	"Mr. Do!'s Castle",
	"Popeye",
	"Q-bert's Qubes [a]",
	"Q-bert's Qubes",
	"Star Wars - Death Star Battle (Parker Bros)",
	"Star Wars - The Arcade Game (Parker Bros)",
	"Super Cobra [b]",
	"Super Cobra",
	"Tooth Protectors (DSD-Camelot)",
	"Tutankham",
	"Popeye_(eks)",
	"Super Cobra_(eks)",
	"Star Wars - Death Star Battle_(eks)",
	"Tutankham_(eks)",
	"Star Wars - The Arcade Game (proto)",
	"Star Wars - The Arcade Game (PAL)",
	"Frogger II - Threedeep! (PAL)",
	"Gyruss (PAL)",
	""
};

dd	BS_4[] = {
	0x584f6777,	/* Espial [b].bin */
	0x8d70fa42,	/* Espial.bin */
	0x8beb03d4,	/* Miner 2049er [b1].bin */
	0x33f2856f,	/* Miner 2049er [b2].bin */
	0xf859122e,	/* Miner 2049er Vol. 2 [b1].bin */
	0x281a1ca1,	/* Miner 2049er Vol. 2 [b2].bin */
	0x350c63ba,	/* Miner 2049er Vol. 2.bin */
	0x728b941c,	/* Miner 2049er.bin */
	0x13bf2da3,	/* Polaris [b].bin */
	0x7ce5312e,	/* Polaris.bin */
	0x40706361,	/* River Patrol (Tigervision).bin */
	0x2c34898f,	/* Springer.bin */
	-1
};

char BS_4_DESC[][30] = {
	"Espial [b]",
	"Espial",
	"Miner 2049er [b1]",
	"Miner 2049er [b2]",
	"Miner 2049er Vol. 2 [b1]",
	"Miner 2049er Vol. 2 [b2]",
	"Miner 2049er Vol. 2",
	"Miner 2049er",
	"Polaris [b]",
	"Polaris",
	"River Patrol (Tigervision)",
	"Springer",
	-1
};

dd	BS_5[] = {
	0x7d23e780,	/* Decathlon.bin */
	0xa51c0236,	/* Robot Tank.bin */
    0xd8ecf576, /* Decathlon (PAL) */
    0x0e8757b0, /* Robot Tank (PAL) */
    0x94e8df6b, /* Space Shuttle (PAL) */
	-1
};

char BS_5_DESC[][25] = {
	"Decathlon",
	"Robot Tank",
    "Decathlon (PAL)",
    "Robot Tank (PAL)",
    "Space Shuttle (PAL)",
	-1
};

/*
dd	BS_9[] = {
	0x6a31beac,	 Private Eye (CCE).bin
	0x3fa749c0,	 Private Eye [b].bin
	0x33242242,	 Private Eye.bin
	-1
};
*/

dd	BS_6[] = {
	0xa972c32b,	/* Dig Dug.bin */
	0x66cdb94b,	/* Off the Wall [o].bin */
	0xbd75d92b,	/* Off the Wall.bin */
	-1
};

char BS_6_DESC[][20] = {
	"Dig Dug.bin",
	"Off the Wall [o]",
	"Off the Wall",
	-1
};

dd	BS_7[] = {
	0x8eed6b02,	/* Bump n Jump [b].bin */
	0xd523e776,	/* Bump n Jump.bin */
	0x24c35820,	/* Burgertime.bin */
	0x5c161fe4,	/* Masters of the Universe - The Power of He-Man.bin */
	-1
};

char BS_7_DESC[][50] = {
	"Bump n Jump [b].bin",
	"Bump n Jump.bin",
	"Burgertime.bin",
	"Masters of the Universe - The Power of He-Man",
	-1
};

/*
dd	BS_8[] = {
        0xbe1047cf,      Fatal Run (PAL).bin
	-1
};
*/

dd crc;			/* holds accumulated CRC */
dd crctab[256];		/* table to help CRC calculation */

#define CRC16_REV 0xA001	/* CRC-16 polynomial reversed */
#define CRC32_REV 0xA0000001	/* CRC-32 polynomial reversed */
